(* file: baseConversions.ml
 * author: Bob Muller
 *
 * Some example OCaml functions working with numerals systems.
 *)

let f2i = int_of_float
let div m n = (m / n, m mod n)
	    
(* toDecimal : int -> int list -> int

   The call (toDecimal base digits) returns the decimal
   representation of the numeral represented by the base base
   digits in the list digits. For example, the call 
   (toDecimal 2 [0; 1; 1]) returns 3.
 *)
let toDecimal base digits =
  let digits = List.rev digits in
  let rec repeat power digits acc =
    match digits with
    | [] -> acc
    | digit::digits ->
       let newAcc = acc + (digit * (f2i ((float base) ** (float power))))
       in
       repeat (power + 1) digits newAcc
  in
  repeat 0 digits 0


(* decimalTo : int -> int -> int list

   The call (decimalTo n base) returns the list of base base
   digits representing the decimal numeral n. It is the case
   that:

   (toDecimal base (decimalTo n base)) = n for any decimal
   numeral n.
 *)
let decimalTo n base =
  let rec repeat n acc =
    match n = 0 with
    | true  -> acc
    | false ->
       let (q, r) = div n base
       in
       repeat q (r::acc)
  in
  repeat n []

